@@ -17,16 +17,16 @@ enum {
 };
 
 struct nftnl_expr *nftnl_expr_alloc(const char *name);
-void nftnl_expr_free(struct nftnl_expr *expr);
+void nftnl_expr_free(const struct nftnl_expr *expr);
 
 bool nftnl_expr_is_set(const struct nftnl_expr *expr, uint16_t type);
-void nftnl_expr_set(struct nftnl_expr *expr, uint16_t type, const void *data, uint32_t data_len);
+int nftnl_expr_set(struct nftnl_expr *expr, uint16_t type, const void *data, uint32_t data_len);
 #define nftnl_expr_set_data nftnl_expr_set
 void nftnl_expr_set_u8(struct nftnl_expr *expr, uint16_t type, uint8_t data);
 void nftnl_expr_set_u16(struct nftnl_expr *expr, uint16_t type, uint16_t data);
 void nftnl_expr_set_u32(struct nftnl_expr *expr, uint16_t type, uint32_t data);
 void nftnl_expr_set_u64(struct nftnl_expr *expr, uint16_t type, uint64_t data);
-void nftnl_expr_set_str(struct nftnl_expr *expr, uint16_t type, const char *str);
+int nftnl_expr_set_str(struct nftnl_expr *expr, uint16_t type, const char *str);
 
 const void *nftnl_expr_get(const struct nftnl_expr *expr, uint16_t type, uint32_t *data_len);
 #define nftnl_expr_get_data nftnl_expr_get
@@ -36,13 +36,26 @@ uint32_t nftnl_expr_get_u32(const struct nftnl_expr *expr, uint16_t type);
 uint64_t nftnl_expr_get_u64(const struct nftnl_expr *expr, uint16_t type);
 const char *nftnl_expr_get_str(const struct nftnl_expr *expr, uint16_t type);
 
-int nftnl_expr_snprintf(char *buf, size_t buflen, struct nftnl_expr *expr, uint32_t type, uint32_t flags);
+bool nftnl_expr_cmp(const struct nftnl_expr *e1, const struct nftnl_expr *e2);
+
+int nftnl_expr_snprintf(char *buf, size_t buflen, const struct nftnl_expr *expr, uint32_t type, uint32_t flags);
 
 enum {
 	NFTNL_EXPR_PAYLOAD_DREG	= NFTNL_EXPR_BASE,
 	NFTNL_EXPR_PAYLOAD_BASE,
 	NFTNL_EXPR_PAYLOAD_OFFSET,
 	NFTNL_EXPR_PAYLOAD_LEN,
+	NFTNL_EXPR_PAYLOAD_SREG,
+	NFTNL_EXPR_PAYLOAD_CSUM_TYPE,
+	NFTNL_EXPR_PAYLOAD_CSUM_OFFSET,
+	NFTNL_EXPR_PAYLOAD_FLAGS,
+};
+
+enum {
+	NFTNL_EXPR_NG_DREG	= NFTNL_EXPR_BASE,
+	NFTNL_EXPR_NG_MODULUS,
+	NFTNL_EXPR_NG_TYPE,
+	NFTNL_EXPR_NG_OFFSET,
 };
 
 enum {
@@ -51,12 +64,24 @@ enum {
 	NFTNL_EXPR_META_SREG,
 };
 
+enum {
+	NFTNL_EXPR_RT_KEY	= NFTNL_EXPR_BASE,
+	NFTNL_EXPR_RT_DREG,
+};
+
 enum {
 	NFTNL_EXPR_CMP_SREG	= NFTNL_EXPR_BASE,
 	NFTNL_EXPR_CMP_OP,
 	NFTNL_EXPR_CMP_DATA,
 };
 
+enum {
+	NFTNL_EXPR_RANGE_SREG	= NFTNL_EXPR_BASE,
+	NFTNL_EXPR_RANGE_OP,
+	NFTNL_EXPR_RANGE_FROM_DATA,
+	NFTNL_EXPR_RANGE_TO_DATA,
+};
+
 enum {
 	NFTNL_EXPR_IMM_DREG	= NFTNL_EXPR_BASE,
 	NFTNL_EXPR_IMM_DATA,
@@ -104,6 +129,7 @@ enum {
 	NFTNL_EXPR_LOOKUP_DREG,
 	NFTNL_EXPR_LOOKUP_SET,
 	NFTNL_EXPR_LOOKUP_SET_ID,
+	NFTNL_EXPR_LOOKUP_FLAGS,
 };
 
 enum {
@@ -150,6 +176,9 @@ enum {
 enum {
 	NFTNL_EXPR_LIMIT_RATE	= NFTNL_EXPR_BASE,
 	NFTNL_EXPR_LIMIT_UNIT,
+	NFTNL_EXPR_LIMIT_BURST,
+	NFTNL_EXPR_LIMIT_TYPE,
+	NFTNL_EXPR_LIMIT_FLAGS,
 };
 
 enum {
@@ -161,10 +190,19 @@ enum {
 	NFTNL_EXPR_QUEUE_NUM	= NFTNL_EXPR_BASE,
 	NFTNL_EXPR_QUEUE_TOTAL,
 	NFTNL_EXPR_QUEUE_FLAGS,
+	NFTNL_EXPR_QUEUE_SREG_QNUM,
 };
 
 enum {
-	NFTNL_EXPR_MASQ_FLAGS	= NFTNL_EXPR_BASE,
+	NFTNL_EXPR_QUOTA_BYTES	= NFTNL_EXPR_BASE,
+	NFTNL_EXPR_QUOTA_FLAGS,
+	NFTNL_EXPR_QUOTA_CONSUMED,
+};
+
+enum {
+	NFTNL_EXPR_MASQ_FLAGS		= NFTNL_EXPR_BASE,
+	NFTNL_EXPR_MASQ_REG_PROTO_MIN,
+	NFTNL_EXPR_MASQ_REG_PROTO_MAX,
 };
 
 enum {
@@ -173,6 +211,38 @@ enum {
 	NFTNL_EXPR_REDIR_FLAGS,
 };
 
+enum {
+	NFTNL_EXPR_DUP_SREG_ADDR = NFTNL_EXPR_BASE,
+	NFTNL_EXPR_DUP_SREG_DEV,
+};
+
+enum {
+	NFTNL_EXPR_FWD_SREG_DEV = NFTNL_EXPR_BASE,
+};
+
+enum {
+	NFTNL_EXPR_HASH_SREG	= NFTNL_EXPR_BASE,
+	NFTNL_EXPR_HASH_DREG,
+	NFTNL_EXPR_HASH_LEN,
+	NFTNL_EXPR_HASH_MODULUS,
+	NFTNL_EXPR_HASH_SEED,
+	NFTNL_EXPR_HASH_OFFSET,
+};
+
+enum {
+	NFTNL_EXPR_FIB_DREG	= NFTNL_EXPR_BASE,
+	NFTNL_EXPR_FIB_RESULT,
+	NFTNL_EXPR_FIB_FLAGS,
+};
+
+enum {
+	NFTNL_EXPR_OBJREF_IMM_TYPE	= NFTNL_EXPR_BASE,
+	NFTNL_EXPR_OBJREF_IMM_NAME,
+	NFTNL_EXPR_OBJREF_SET_SREG,
+	NFTNL_EXPR_OBJREF_SET_NAME,
+	NFTNL_EXPR_OBJREF_SET_ID,
+};
+
 /*
  * Compat
  */
@@ -211,6 +281,9 @@ enum {
 	NFT_EXPR_PAYLOAD_BASE,
 	NFT_EXPR_PAYLOAD_OFFSET,
 	NFT_EXPR_PAYLOAD_LEN,
+	NFT_EXPR_PAYLOAD_SREG,
+	NFT_EXPR_PAYLOAD_CSUM_TYPE,
+	NFT_EXPR_PAYLOAD_CSUM_OFFSET,
 };
 
 enum {
@@ -219,6 +292,11 @@ enum {
 	NFT_EXPR_META_SREG,
 };
 
+enum {
+	NFT_EXPR_RT_KEY		= NFT_RULE_EXPR_ATTR_BASE,
+	NFT_EXPR_RT_DREG,
+};
+
 enum {
 	NFT_EXPR_CMP_SREG	= NFT_RULE_EXPR_ATTR_BASE,
 	NFT_EXPR_CMP_OP,
